"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = __importDefault(require("lodash"));
const irc_1 = __importDefault(require("irc"));
class Bot {
    constructor(conf, logger) {
        this.commands = [];
        this.delayedChannels = [];
        this.msgs = [];
        this.registered = false;
        this.config = conf;
        this.logger = logger;
        this.logger.info('Initializing...');
        if (process.env.TARGET && process.env.MESSAGE) {
            const target = process.env.TARGET;
            const message = process.env.MESSAGE;
            this.config.connectCommands.push({ target, message });
        }
        this.client = new irc_1.default.Client(this.config.server, this.config.nick, this.config.clientOptions);
    }
    joinChannels(channels) {
        if (!lodash_1.default.isUndefined(channels)) {
            if (this.registered) {
                channels.forEach((channel) => {
                    this.client.join(channel);
                });
            }
            else {
                this.delayedChannels = this.delayedChannels.concat(channels);
            }
        }
    }
    registeredCallback({ server }) {
        this.logger.info(`Connected to server ${server}`);
        this.registered = true;
        if (this.config.connectCommands) {
            this.config.connectCommands.forEach(({ target, message }) => {
                if (target && message)
                    this.client.say(target, message);
            });
        }
        if (this.delayedChannels)
            this.joinChannels(this.delayedChannels);
    }
    joinCallback(channel, nick) {
        this.logger.info(`Joined ${channel} as ${nick}`);
        if (channel in this.config.joinCommands) {
            this.config.joinCommands[channel].forEach(({ target, message }) => {
                if (target && message)
                    this.client.say(target, message);
            });
        }
    }
    connect() {
        this.logger.info(`Connecting to ${this.config.server} as ${this.config.nick}...`);
        this.client.addListener('registered', this.registeredCallback);
        this.client.addListener('join', this.joinCallback);
        this.client.addListener('message', this.messageCallback);
        this.client.addListener('error', (message) => this.logger.warn('IRC client error: ', message));
    }
    messageCallback(from, to, text, message) {
        this.logger.debug(`message from ${from} to ${to}: ${text}`);
        const cmdArr = text.trim().match(/^[.|!](\w+)\s?(.*)$/i);
        if (!cmdArr || cmdArr.length <= 1)
            return;
        const command = cmdArr[1].toLowerCase();
        const cmdArgs = cmdArr[2];
        this.logger.debug(cmdArr);
        if (this.config.nick === to) {
            this.msgs.forEach(lodash_1.default.bind((c) => {
                if (command === c.cmd) {
                    this.logger.debug(`command: ${c.cmd}`);
                    c.callback(this.client, message, cmdArgs);
                }
            }, this));
        }
        else {
            this.commands.forEach(lodash_1.default.bind((c) => {
                if (command === c.cmd) {
                    if ((lodash_1.default.includes(c.channel, to) || c.channel === 'all') &&
                        (lodash_1.default.isUndefined(c.exclude) || !lodash_1.default.includes(c.exclude, to))) {
                        this.logger.debug(`command: ${c.cmd}`);
                        c.callback(this.client, message, cmdArgs);
                    }
                }
            }, this));
        }
    }
    registerPlugin(PluginClass, conf) {
        const plugin = new PluginClass(conf, this.logger);
        this.joinChannels(plugin.config.pluginOptions.channels);
        Object.entries(plugin.commands).forEach(([command, func]) => {
            this.cmd(command, plugin.config.pluginOptions.channels, plugin.config.pluginOptions.channelsToExclude, func);
        });
        Object.entries(plugin.privateCommands).forEach(([command, func]) => {
            this.msg(command, func);
        });
    }
    cmd(cmd, channel, exclude, callback) {
        this.commands.push({ cmd, channel, exclude, callback });
    }
    msg(cmd, callback) {
        this.msgs.push({ cmd, callback });
    }
}
exports.default = Bot;
